#ident "@(#)$Header$"
##############################################################################
# Copyright (C) Fujitsu Technology Solutions 2009
# All rights reserved
# RPM spec file for the Fujitsu Technology Solutions ServerView binary kernel modules
##############################################################################

Summary      : Fujitsu Technology Solutions ServerView Server Software
Name         : srvmagt
@Version@
@Release@
# Copyright    : Copyright (C) Fujitsu Technology Solutions 2009
License      : GPL
Group        : Applications/Management
URL          : http://serverview.abg.fsc.net/
Distribution : Fujitsu Technology Solutions ServerStart
Vendor       : Fujitsu Technology Solutions

#-----------------------------------------------------------------------------
%description
#-----------------------------------------------------------------------------
The ServerView Agent Linux packages contain the Environment and Enclosure
Control Daemon eecd that takes care of handling enclosure control data such as
overtemperature or fan failures.

##############################################################################
%package mods_bin
##############################################################################

Summary      : Fujitsu Technology Solutions ServerView Module Binaries
Group        : Applications/Management
Provides     : eecd-modules
Conflicts    : srvmagt-mods_src
AutoReqProv  : no

#-----------------------------------------------------------------------------
%description mods_bin
#-----------------------------------------------------------------------------
The ServerView Agent Linux srvmagt-mods_bin package contains the binary
kernel modules needed by the ServerView Environment and Enclosure Control
Daemon (eecd). They were compiled on a properly equipped reference system
for use on an equivalent but less equipped target system.
@ToolsUsed@

#-----------------------------------------------------------------------------
%files mods_bin
#-----------------------------------------------------------------------------
@Filelist@

#-----------------------------------------------------------------------------
%pre mods_bin
[ "$SRVMAGT_LOG" ] && echo "Running pre ($1) for %{name}-%{version}-%{release}"
#-----------------------------------------------------------------------------

export REF_KERNEL="%{ref_kernel}"
export REF_ARCH="%{build_arch}"
export TGT_KERNEL=`uname -r`
export TGT_ARCH=`uname -m`

[ "$REF_ARCH" != "$TGT_ARCH" ] && {
	echo "The target architecture $TGT_ARCH does not match the reference
architecture $REF_ARCH where the modules were compiled."
	exit 1
}

[ "$REF_KERNEL" != "$TGT_KERNEL" ] && {
	echo "The target kernel $TGT_KERNEL does not match the reference kernel
$REF_KERNEL where the modules were compiled."
	exit 1
}

exit 0

#-----------------------------------------------------------------------------
%post mods_bin
[ "$SRVMAGT_LOG" ] && echo "Running post ($1) for %{name}-%{version}-%{release}"
#-----------------------------------------------------------------------------

# Check if we operate on separate root (--root switch)

ONSYSTEM=
[ -e /proc/1/exe ] && ONSYSTEM=true

RUNNING_SYSTEM=
RUNNING_ESX=
if [ -f /etc/SuSE-release ]; then
	RUNNING_SYSTEM="suse"
elif [ -f /etc/vmware-release ]; then
	# NOTE: Do this BEFORE the Red Hat test!
	# VMware ESX Server 3 (Dali)
	RUNNING_SYSTEM="vmware"
	RUNNING_ESX="3"
elif [ -f /etc/vmware ]; then
	# NOTE: Do this AFTER the ESX 3 test!
	# VMware ESX Server 2.5.4
	RUNNING_SYSTEM="vmware"
	RUNNING_ESX="2.5"
elif [ -f /etc/redhat-release ]; then
	# NOTE: Do this AFTER the VMware tests!
	RUNNING_SYSTEM="redhat"
fi

# Disable EDAC functionality during the lifecycle of ServerView Agents Linux
#
# RHEL5: Only the kernel module which corresponds to the chipset and 
#        edac_mc is blacklisted via the file "fsc-srvmagt" in the 
#        directory /etc/modprobe.d
#            
# RHEL4: From RHEL4 Update3 the EDAC functionality is installed but         
#        disabled by default via the entries in /etc/hotplug/blacklist.
#        During several updates this entries could be lost.
#        If the entries or the blacklist file are missing, we establish
#        the RHEL4 Update 3 default.
#

[ "$ONSYSTEM" ] && [ "$RUNNING_SYSTEM" = "redhat" ] && {
	[ -d "/lib/modules/$(uname -r)/kernel/drivers/edac" ] && {
		edac_mods=$(lsmod | grep edac_mc)
		if [ -n "$edac_mods" ]; then
			edac_cnt=0

			#Assumption: 
			# in the variable "edac_ch" we get the fourth token of
			# lsmod output which represents the chipset dependent
			# module. The use count is a number between 0 and 9
			#	Module                  Size  Used by
			#	edac_mc                24977  1 e752x_edac
			edac_ch=${edac_mods#* [0-9] }

			if [ -d "/etc/modprobe.d" ]; then	# RHEL5
				>/etc/modprobe.d/fsc-srvmagt
				chmod 644 /etc/modprobe.d/fsc-srvmagt
				for mod in "edac_mc" $edac_ch; do
					echo "blacklist $mod" >>/etc/modprobe.d/fsc-srvmagt
				done
				logger -t "%post mods_bin" "Disable and blacklist EDAC modules [edac_mc, $mod] in /etc/modprobe.d/"

			elif [ -f "/etc/hotplug/blacklist" ]; then	# RHEL4

				edac_cnt=$(grep -c edac /etc/hotplug/blacklist)
				if [ $edac_cnt -lt 8 ]; then            # 8 occuences of "edac" per default
					sed -e /edac/d /etc/hotplug/blacklist >/tmp/blacklist
					echo "#disable edac" >>/tmp/blacklist
					for mod in $(ls /lib/modules/$(uname -r)/kernel/drivers/edac); do
						echo $(basename $mod .ko) >>/tmp/blacklist
					done
					cat /tmp/blacklist >/etc/hotplug/blacklist

					logger -t "%post mods_bin" "Disable and blacklist all EDAC modules in /etc/hotplug/blacklist"
				fi
			fi
		else
			logger -t "%post mods_bin" "EDAC module installed but not enabled. Not blacklisted"
		fi
	}
}

# Create /dev/pci if it doesn't exist
# Insert my init script into automatic boot startup

[ ! -e /dev/pci ] && /usr/bin/install -d -m 755 -o root -g root /dev/pci

if [ "$ONSYSTEM"  -a "$RUNNING_SYSTEM" = "suse" ]
then
	/sbin/insserv eecd_mods_bin
else
	/sbin/chkconfig --add eecd_mods_bin
fi

# Start modules

[ "$SRVMAGT_LOG" ] && echo "Load my modules"
[ "$ONSYSTEM" ] && [ -x /etc/init.d/eecd_mods_bin ] && /etc/init.d/eecd_mods_bin start

exit 0

#-----------------------------------------------------------------------------
%preun mods_bin
[ "$SRVMAGT_LOG" ] && echo "Running preun ($1) for %{name}-%{version}-%{release}"
#-----------------------------------------------------------------------------

ONSYSTEM=
[ -e /proc/1/exe ] && ONSYSTEM=true

[ -x /usr/sbin/eecd ] && {
	echo -e "Cannot erase %{name}-mods_bin-%{version}-%{release}. Please erase srvmagt-eecd package first!! \n"
	exit -1
}

[ "$SRVMAGT_LOG" ] && echo "Remove my modules and devices and remove init script "

[ "$ONSYSTEM" ] && [ -x /etc/init.d/eecd_mods_bin ] && /etc/init.d/eecd_mods_bin stop

[ $1 -eq 0 ] && {
	if [ "$ONSYSTEM"  -a "$RUNNING_SYSTEM" = "suse" ]
	then
		/sbin/insserv -r eecd_mods_bin
	else
		/sbin/chkconfig --del eecd_mods_bin
	fi
}

exit 0

#-----------------------------------------------------------------------------
%postun mods_bin
[ "$SRVMAGT_LOG" ] && echo "Running postun ($1) for %{name}-%{version}-%{release}"
#-----------------------------------------------------------------------------

# Check if we operate on separate root (--root switch)

ONSYSTEM=
[ -e /proc/1/exe ] && ONSYSTEM=true

RUNNING_SYSTEM=
RUNNING_ESX=
if [ -f /etc/SuSE-release ]; then
	RUNNING_SYSTEM="suse"
elif [ -f /etc/vmware-release ]; then
	# NOTE: Do this BEFORE the Red Hat test!
	# VMware ESX Server 3 (Dali)
	RUNNING_SYSTEM="vmware"
	RUNNING_ESX="3"
elif [ -f /etc/vmware ]; then
	# NOTE: Do this AFTER the ESX 3 test!
	# VMware ESX Server 2.5.4
	RUNNING_SYSTEM="vmware"
	RUNNING_ESX="2.5"
elif [ -f /etc/redhat-release ]; then
	# NOTE: Do this AFTER the VMware tests!
	RUNNING_SYSTEM="redhat"
fi


# Unload and delete the modules and the environment used to build my binary
# package.
# distinguish between upgrading and removing

if [ "$1" -eq 0 ]; then

	[ "$SRVMAGT_LOG" ] && echo "Unload smbus module and remove device"
	[ "$ONSYSTEM" ] && /sbin/rmmod smbus 2>/dev/null >/dev/null
	/bin/rm -f /dev/pci/smbus

	/bin/rm -rf /lib/modules/*/srvmagt
fi

# Reenable EDAC functionality 

[ "$ONSYSTEM" ] && [ "$RUNNING_SYSTEM" = "redhat" ] && {
	[ -d "/lib/modules/$(uname -r)/kernel/drivers/edac" ] && {
		[ -d "/etc/modprobe.d" ] && {
			rm -f /etc/modprobe.d/fsc-srvmagt
			logger -t "%postun mods_bin" "EDAC modules blacklisted by Fujitsu Technology Solutions ServerView Agents reenabled"
		}
	}
}

exit 0

##############################################################################
# EOF
##############################################################################
